package eu.clarin.cmdi.mdservice.action;

import java.io.BufferedReader;
import java.io.ByteArrayInputStream;
import java.io.IOException;
import java.io.InputStream;
import java.io.InputStreamReader;
import java.io.StringWriter;
import java.io.UnsupportedEncodingException;
import java.util.HashMap;
import java.util.Map;

import javax.xml.parsers.ParserConfigurationException;
import javax.xml.transform.TransformerConfigurationException;
import javax.xml.transform.TransformerException;
import javax.xml.transform.TransformerFactory;
import javax.xml.transform.TransformerFactoryConfigurationError;
import javax.xml.transform.dom.DOMSource;
import javax.xml.transform.stream.StreamResult;

import org.w3c.dom.Document;
import org.w3c.dom.Node;

import eu.clarin.cmdi.mdservice.internal.Cache;
import eu.clarin.cmdi.mdservice.internal.Utils;
import eu.clarin.cmdi.mdservice.model.Diagnostic;
import eu.clarin.cmdi.mdservice.model.Query;
import eu.clarin.cmdi.mdservice.model.WorkspaceProfile;
import eu.clarin.cmdi.mdservice.proxy.FCSProxy;
import eu.clarin.cmdi.mdservice.proxy.MDRepoProxy;
import eu.clarin.cmdi.mdservice.proxy.Pz2Proxy;
import eu.clarin.cmdi.mdservice.proxy.SRUProxy;




public class FCSAction extends SRUAction 
{

	private static final long serialVersionUID = 1L;
	
	/** 
	 * Properties to be filled by Struts with according request-parameters
	 */	
	/*
	private String q;
	private String version;
	private String operation;
	private String query;
	private String scanClause;
	private int startRecord;
	private int maximumRecords;
	private String recordPacking;
	private String recordSchema;
	private String recordXPath;
	private int resultSetTTL;
	private String sortKeys;
	private String stylesheet;	
	private String x_cmd_repository;
	private String x_cmd_collections;
	private Set paramSet;
*/
	private String x_content;
	private Map<String,String[]> fcsparams;
	
	private static int OP_SEARCHRETRIEVE = 1;
	private static int OP_EXPLAIN = 2;
	private static int OP_SCAN = 3;
	
	
	//TODO cache after analysis ?
	@Override
	public String getCache() {
		return Cache.SKIP;
	}
	
	/*
	@Override
	public String getRepository() {
		return getParam("x-content");
	}
	*/
	 
	/**
	 * primitive identification of the target-proxy
	 * base for finding the right base_url in the props
	 * subclass has to override with its specific proxykey
	 * @return
	 */
	public String getProxyKey() {
		return "";
	}

	public void addFCSParam(String key, String value){
		String[] sarr = new String[1];
		sarr[0] = value;
		fcsparams.put(key, sarr);	
	}
	public Map<String, String[]> getFCSParams(){
		if (fcsparams == null){

			fcsparams = new HashMap<String, String[]>();
			
			if ( params.get("operation") != null){
				addFCSParam("operation",params.get("operation")[0]);
			}
			if ( params.get("scanClause") != null){
				addFCSParam("scanClause",params.get("scanClause")[0]);
			}
			if ( params.get("query") != null){
				addFCSParam("query",params.get("query")[0]);
			}
			if ( params.get("x-context") != null){
				if (WorkspaceProfile.getFCSMapping(params.get("x-context")[0]) != null){
					addFCSParam("x-context",params.get("x-context")[0]);
				}
				
			}
		
		}
		
		return fcsparams;
		
	}
	
	@Override
	protected void setDefaultParams(){
		// set defaults
		
		if ( params.get("version") == null){
			addParam("version","1.2");
		}
		/*
		if ( params.get("operation") == null){
			addParam("operation","searchRetrieve");
		}
		//q=query			
		if (getQ() != null) {
			addParam("q",getQ());
			addParam("query",getQ());
		} else {
			addParam("q",this.getParam("query"));
		}
		
		if ( params.get("startRecord") == null){
			addParam("startRecord","1");
		}
		if ( params.get("maximumRecords") == null){
			addParam("maximumRecords","10");
		}
		
		if ( params.get("x-cmd-maxdepth") == null){
			if (params.get("maxdepth") == null){
				addParam("maxdepth","2");
			} 
		} else {
			addParam("maxdepth",params.get("x-cmd-maxdepth")[0]);
		}
		*/
		if ( params.get("x-cmd-repository") == null){
			if (params.get("repository") == null){
				if (params.get("x-context") == null){
					addParam("repository",WorkspaceProfile.getRepositoryByIndex(0));
				} else {
					// search in fcs-mapping
					if (WorkspaceProfile.getFCSMapping(params.get("x-context")[0]) == null){
						addParam("repository",params.get("x-context")[0]);
					} else {
						addParam("repository",WorkspaceProfile.getFCSMapping(params.get("x-context")[0]));
					}
					//addParam("repository",params.get("x-context")[0]);
				}
			} 
		} else {
			addParam("repository",params.get("x-cmd-repository")[0]);
		}
		/*
		if ( params.get("x-cmd-cache") == null){
			if (params.get("cache") == null){
				addParam("cache",Cache.SKIP);
			} 
		} else {
			addParam("cache",params.get("x-cmd-cache")[0]);
		}
		
		if ( params.get("x-cmd-lang") == null){
			if (params.get("lang") == null){
				addParam("lang","en");
			} 
		} else {
			addParam("lang",params.get("x-cmd-lang")[0]);
		}
		*/
		if ( params.get("x-cmd-format") == null){
			if (params.get("format") == null){
				if (this.getParam("format") == null) {
					addParam("format","xml");
				} else if (this.getParam("format").equals("")){
					addParam("format","xml");
				} else {
					addParam("format",this.getParam("format"));
				}
			} 
		} else {
			addParam("format",params.get("x-cmd-format")[0]);
		}

		addParam("fullformat",getFullFormat());
	}
	@Override
	public void setTargetProxy(){
		
		//Query query = new Query();
		switch (WorkspaceProfile.RepositoryType.toRepositoryType(WorkspaceProfile.getRepositoryType(getRepository())))
		{		
		    case PAZPAR: 
		    	setTargetProxy(new Pz2Proxy(new Query(getActionkey(), this.getParams())));
		    	return;
		    case SRU:
		    	setTargetProxy(new SRUProxy(new Query(Query.SRUEXTERN,getParams())));
		    	return;
		    case MD:
		    	setTargetProxy(new MDRepoProxy(new Query(Query.RECORDSET,getParams())));
		    	return;	
		    case FCS:
		    	setTargetProxy(new FCSProxy(new Query(Query.FCSRECORDSET,getFCSParams())));
		    	return;
		    case FCSRESOURCE:
		    	setTargetProxy(new FCSProxy(new Query(Query.FCSRECORDSET,getFCSParams())));
		    	return;
		    default:
		    	//TODO error  = bad workspaceProfile configuration
		    	getDiagnostics().Add(Diagnostic.SYSTEM_ERROR, "Error in workspaceprofile configuration", "repository=" + getRepository() );
		}
}
}